@@ -31,6 +31,7 @@
 #include "check_impl.h"
 #include "check_msg.h"
 #include "check_pack.h"
+#include "check_str.h"
 
 
 /* 'Pipe' is implemented as a temporary file to overcome message
@@ -54,7 +55,9 @@
  */
 
 static FILE *send_file1;
+static char *send_file1_name;
 static FILE *send_file2;
+static char *send_file2_name;
 
 static FILE * get_pipe(void);
 static void setup_pipe (void);
@@ -121,6 +124,12 @@ TestResult *receive_test_result (int waserror)
     eprintf ("Error in call to get_pipe",__FILE__, __LINE__ - 2);
   rewind(fp);
   rmsg = punpack (fileno(fp));
+
+  if(rmsg == NULL)
+  {
+	  eprintf ("Error in call to punpack",__FILE__, __LINE__ - 4);
+  }
+
   teardown_pipe();
   setup_pipe();
 
@@ -155,7 +164,7 @@ static TestResult *construct_test_result (RcvMsg *rmsg, int waserror)
   tr = tr_create();
 
   if (rmsg->msg != NULL || waserror) {
-    tr->ctx = (cur_fork_status () == CK_FORK) ? rmsg->lastctx : rmsg->failctx;
+    tr->ctx = rmsg->lastctx;
     tr->msg = rmsg->msg;
     rmsg->msg = NULL;
     tr_set_loc_by_ctx (tr, tr->ctx, rmsg);
@@ -183,25 +192,49 @@ void teardown_messaging(void)
   teardown_pipe();
 }
 
-static FILE *
-open_tmp_file (void)
+/**
+ * Open a temporary file.
+ *
+ * If the file could be unlinked upon creation, the name
+ * of the file is not returned via 'name'. However, if the
+ * file could not be unlinked, the name is returned,
+ * expecting the caller to both delete the file and
+ * free the 'name' field after the file is closed.
+ */
+static FILE * open_tmp_file (char ** name)
 {
-  /* workaround from Bug 3314868, but maybe tmpfile works if TMPDIR is set */
+  FILE *file;
+  *name = NULL;
+
+  /* Windows does not like tmpfile(). This is likely because tmpfile()
+   * call unlink() on the file before returning it, to make sure the
+   * file is deleted when it is closed. The unlink() call also fails
+   * on Windows if the file is still open. */
   /* also note that mkstemp is apparently a C90 replacement for tmpfile */
   /* perhaps all we need to do on Windows is set TMPDIR to whatever is
      stored in TEMP for tmpfile to work */
-  /* it's also not clear if we need _tempnam instead of tempnam on WIN32 */
   /* and finally, the "b" from "w+b" is ignored on OS X, not sure about WIN32 */
-  FILE *file = tmpfile ();
-#ifdef WIN32
+
+  file = tmpfile ();
   if (file == NULL)
     {
       char *tmp = getenv ("TEMP");
-      char *tmp_file = _tempnam (tmp, "check_");
-      file = fopen (tmp_file, "w+b");
+      char *tmp_file = tempnam (tmp, "check_");
+      /*
+       * Note, tempnam is not enough to get a unique name. Between
+       * getting the name and opening the file, something else also
+       * calling tempnam() could get the same name. It has been observed
+       * on MinGW-w64 builds on Wine that this exact thing happens
+       * if multiple instances of a unit tests are running concurrently.
+       * To prevent two concurrent unit tests from getting the same file,
+       * we append the pid to the file. The pid should be unique on the
+       * system.
+       */
+      char *uniq_tmp_file = ck_strdup_printf("%s.%d",tmp_file, getpid());
+      file = fopen (uniq_tmp_file, "w+b");
+      *name = uniq_tmp_file;
       free (tmp_file);
     }
-#endif /* WIN32 */
   return file;
 }
 
@@ -210,12 +243,12 @@ setup_pipe (void)
 {
   if (send_file1 == NULL)
     {
-      send_file1 = open_tmp_file ();
+      send_file1 = open_tmp_file (&send_file1_name);
       return;
     }
   if (send_file2 == NULL)
     {
-      send_file2 = open_tmp_file ();
+      send_file2 = open_tmp_file (&send_file2_name);
       return;
     }
   eprintf ("Only one nesting of suite runs supported", __FILE__, __LINE__);
@@ -226,9 +259,19 @@ static void teardown_pipe(void)
   if (send_file2 != 0) {
     fclose(send_file2);
     send_file2 = 0;
+    if(send_file2_name != NULL) {
+		unlink(send_file2_name);
+		free(send_file2_name);
+		send_file2_name = NULL;
+	}
   } else if (send_file1 != 0) {
     fclose(send_file1);
     send_file1 = 0;
+    if(send_file1_name != NULL) {
+		unlink(send_file1_name);
+		free(send_file1_name);
+		send_file1_name = NULL;
+	}
   } else {
     eprintf("No messaging setup", __FILE__, __LINE__);
   }
