@@ -24,10 +24,6 @@
 #include <string.h>
 #include <stdio.h>
 
-#ifdef HAVE_STDINT_H
-#include <stdint.h>
-#endif
-
 #include "check.h"
 #include "check_error.h"
 #include "check_list.h"
@@ -288,7 +284,7 @@ static void check_type (int type, const char *file, int line)
 
 #ifdef HAVE_PTHREAD
 static pthread_mutex_t ck_mutex_lock = PTHREAD_MUTEX_INITIALIZER;
-void ppack_cleanup( void *mutex )
+static void ppack_cleanup( void *mutex )
 {
   pthread_mutex_unlock(mutex);
 }
