@@ -27,7 +27,9 @@ static char signal_8_str[SIG_STR_LEN];
 /* FIXME: all these line numbers are kind of hard to maintain */
 static master_test_t master_tests[] = {
   { "Simple Tests", CK_FAILURE, "Failure expected" },
+#ifdef HAVE_FORK
   { "Simple Tests", CK_ERROR,   "Early exit with return value 1" },
+#endif
   { "Simple Tests", CK_PASS,    "Passed" },
   { "Simple Tests", CK_FAILURE, "This test should fail" },
   { "Simple Tests", CK_PASS,    "Passed" },
@@ -69,6 +71,7 @@ static master_test_t master_tests[] = {
   { "Simple Tests", CK_FAILURE, "Assertion 'x==y' failed: x==0x1, y==0x2" },
   { "Simple Tests", CK_FAILURE, "Assertion 'x!=z' failed: x==0x1, z==0x1" },
   
+#ifdef HAVE_FORK
   { "Signal Tests", CK_ERROR,   signal_11_str },
   { "Signal Tests", CK_PASS,    "Passed" },
   { "Signal Tests", CK_ERROR,   signal_11_8_str },
@@ -76,8 +79,9 @@ static master_test_t master_tests[] = {
   { "Signal Tests", CK_FAILURE, "Early exit with return value 1" },
   { "Signal Tests", CK_ERROR,   signal_8_str },
   { "Signal Tests", CK_ERROR,   signal_8_str },
+#endif
 
-#if TIMEOUT_TESTS_ENABLED
+#if TIMEOUT_TESTS_ENABLED && defined(HAVE_FORK)
 #if HAVE_WORKING_SETENV
   { "Environment Integer Timeout Tests", CK_ERROR,  "Test timeout expired" },
   { "Environment Integer Timeout Tests", CK_PASS,   "Passed" },
@@ -92,7 +96,7 @@ static master_test_t master_tests[] = {
   { "Environment Double Timeout Tests", CK_ERROR,  "Test timeout expired" },
   { "Environment Double Timeout Tests", CK_ERROR,  "Test timeout expired" },
   { "Environment Double Timeout Tests", CK_ERROR,  "Test timeout expired" },
-#endif
+#endif /* HAVE_WORKING_SETENV */
 
   { "Default Timeout Tests", CK_ERROR,  "Test timeout expired" },
 #ifdef HAVE_LIBRT
@@ -185,19 +189,23 @@ static master_test_t master_tests[] = {
   { "User Double Timeout Scaling Tests", CK_ERROR,  "Test timeout expired" },
   { "User Double Timeout Scaling Tests", CK_ERROR,  "Test timeout expired" },
   { "User Double Timeout Scaling Tests", CK_ERROR,  "Test timeout expired" },
-#endif
-#endif
+#endif /* HAVE_WORKING_SETENV */
+#endif /* TIMEOUT_TESTS_ENABLED && defined(HAVE_FORK) */
 
+#ifdef HAVE_FORK
   { "Limit Tests", CK_ERROR,   "Early exit with return value 1" },
+#endif /* HAVE_FORK */
   { "Limit Tests", CK_FAILURE, "Completed properly" },
   { "Limit Tests", CK_FAILURE, "Completed properly" },
 
+#ifdef HAVE_FORK
   { "Msg and fork Tests", CK_PASS,       "Passed" },
   { "Msg and fork Tests", CK_FAILURE,    "Expected fail" },
   { "Msg and fork Tests", CK_PASS,       "Passed" },
   { "Msg and fork Tests", CK_FAILURE,    "Expected fail" },
   { "Msg and fork Tests", CK_PASS,       "Passed" },
   { "Msg and fork Tests", CK_FAILURE,    "Expected fail" },
+#endif
 
   { "Core", CK_PASS,    "Passed" },
   { "Core", CK_FAILURE, "We failed" }
@@ -411,8 +419,11 @@ Suite *make_master_suite (void)
      preservation of fixture values across tests, regardless
      of the order in which tests are added to the test case */
   tcase_add_test (tc_fixture, test_setup);
+#ifdef HAVE_FORK
+  /* The remaining test runs only work if fork() is available. */
   tcase_add_test (tc_fixture, test_setup);
   tcase_add_test (tc_fixture, test_setup);
+#endif /* HAVE_FORK */
   suite_add_tcase (s, tc_fixture);
   tc_post_teardown = tcase_create ("Fixture Teardown Tests");
   tcase_add_test (tc_post_teardown, test_teardown);
@@ -445,6 +456,7 @@ void setup (void)
   init_signal_strings();
 
   srunner_add_suite(sr, make_sub2_suite());
+
   srunner_run_all(sr, CK_VERBOSE);
   tr_fail_array = srunner_failures(sr);
   tr_all_array = srunner_results(sr);
