@@ -17,11 +17,17 @@ START_TEST(test_fail)
 }
 END_TEST
 
+/*
+ * This test will fail without fork, as it will result in the
+ * unit test runniner exiting early.
+ */
+#if defined(HAVE_FORK)
 START_TEST(test_exit)
 {
   exit(1);
 }
 END_TEST
+#endif /* HAVE_FORK */
 
 START_TEST(test_pass2)
 {
@@ -39,7 +45,9 @@ static Suite *make_s1_suite (void)
   suite_add_tcase(s, tc);
   tcase_add_test (tc, test_pass);
   tcase_add_test (tc, test_fail);
+#if defined(HAVE_FORK)
   tcase_add_test (tc, test_exit);
+#endif /* HAVE_FORK */
 
   return s;
 }
@@ -70,7 +78,7 @@ static void run_tests (int printmode)
 
 static void usage(void)
 {
-  printf ("Usage: ex_output (CRSILENT | CRMINIMAL | CRNORMAL | CRVERBOSE)\n");
+  printf ("Usage: ex_output (CK_SILENT | CK_MINIMAL | CK_NORMAL | CK_VERBOSE)\n");
 }
 
 int main (int argc, char **argv)
